# -*- coding: utf-8 -*-
from setuphelpers import *

r"""
Usable WAPT package functions: install(), uninstall(), session_setup(), audit(), update_package()

Installation procedure: https://docs.microsoft.com/windows/wsl/install-win10
Troobleshooting installation: https://docs.microsoft.com/windows/wsl/troubleshooting
Kernel Installation procedure: https://aka.ms/wsl2kernel
WSL2: https://aka.ms/wsl2-install

"""
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls
bin_name = "wsl_update_x64.msi"


def install():
    # Specific app values
    app_name = control.name

    # Installing the package
    if windows_version() >= WindowsVersions.Windows10v1903:
        with EnsureWUAUServRunning():

            # Get previous state of WSL
            wsl = run_powershell("Get-WindowsOptionalFeature -Online -FeatureName Microsoft-Windows-Subsystem-Linux")
            wsl_status = wsl.get("State") # State: Disable = 0, Enable = 2

            if wsl_status == 0:
                print("Enabling Windows Feature: Microsoft-Windows-Subsystem-Linux")
                run_powershell("Enable-WindowsOptionalFeature -Online -FeatureName Microsoft-Windows-Subsystem-Linux -NoRestart -All",output_format="text")

        # Updating WSL to WSL2
            # Installing the Kernel WSL Update if needed
            try:
                install_msi_if_needed(bin_name)
            except:
                print("INFO: A reboot is required to install %s" % bin_name)
                return

            try:
                print("Enabling Windows Feature: VirtualMachinePlatform")
                run_powershell("Enable-WindowsOptionalFeature -Online -FeatureName VirtualMachinePlatform -NoRestart -All",output_format="text")
            except:
                print("Unable to install %s, please reboot and make sure that virtualization is enabled in the BIOS" % app_name)

            try:
                print("Configuring %s as default" % app_name)
                run_powershell("wsl --set-default-version 2")
            except:
                print("Unable to configure %s as default" % app_name)

    else:
        print("You need Windows 10 version 1903 minimum for installing WSL 2")

    print("INFO: A reboot is required to use %s" % app_name)


def uninstall():
    # Uninstalling the package
    if windows_version() >= WindowsVersions.Windows10v1709:
        with EnsureWUAUServRunning():
            print("Disabling Windows Feature: Microsoft-Windows-Subsystem-Linux")
            run_powershell("Disable-WindowsOptionalFeature -Online -FeatureName Microsoft-Windows-Subsystem-Linux -NoRestart")
            if windows_version() >= "10.0.19041":
                print("Disabling Windows Feature: VirtualMachinePlatform")
                run_powershell("Disable-WindowsOptionalFeature -Online -FeatureName VirtualMachinePlatform -NoRestart")
